/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2013  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "keysequencer.h"
#include <QKeyEvent>
#include <QtDebug>

KeySequencer::KeySequencer (const QString& labelStr, QWidget *parent)
: QDialog (parent)
{
	Ui_.setupUi (this);
	Ui_.DescLabel_->setText (labelStr);
}

QKeySequence KeySequencer::GetResult () const
{
	return Result_;
}

namespace
{
	void FixCodes (int& code, QKeyEvent *event)
	{
		auto fixSingle = [&code, event] (Qt::KeyboardModifier keyM, Qt::Modifier codeM)
		{
			if (event->modifiers () & keyM)
				code += codeM;
		};

		fixSingle (Qt::ControlModifier, Qt::CTRL);
		fixSingle (Qt::AltModifier, Qt::ALT);
		fixSingle (Qt::ShiftModifier, Qt::SHIFT);
		fixSingle (Qt::MetaModifier, Qt::META);
	}
}

void KeySequencer::keyPressEvent (QKeyEvent *event)
{
	int code = 0;
	FixCodes (code, event);

	const int key = event->key ();
	if (key != Qt::Key_Control &&
			key != Qt::Key_Alt &&
			key != Qt::Key_Meta &&
			key != Qt::Key_Shift)
		code += key;

	const QKeySequence ts (code);

	Ui_.Shortcut_->setText (ts.toString ());
	QDialog::keyPressEvent (event);
}

void KeySequencer::keyReleaseEvent (QKeyEvent *event)
{
	const int key = event->key ();
	if (key == Qt::Key_Control ||
			key == Qt::Key_Alt ||
			key == Qt::Key_Meta ||
			key == Qt::Key_Shift)
	{
		reject ();
		return;
	}

	int code = 0;
	FixCodes (code, event);

	if (key != Qt::Key_Control &&
			key != Qt::Key_Alt &&
			key != Qt::Key_Meta &&
			key != Qt::Key_Shift)
		code += key;

	Result_ = QKeySequence (code);

	accept ();
	QDialog::keyReleaseEvent (event);
}

